

def latex_for_figure(ap, name, ftype):
    str_parts = ["",
                    "\\begin{figure}[H]",
                    "\centering",
                    "\\includegraphics{%s/%s%s}" % (ap.FIGURE_FOLDER, name, ftype),
                    "\\caption{%s \label{fig: %s}}" % (name.replace("_", " "), name),
                    "\\end{figure}"
                 ]
    return "\n".join(str_parts)




from pypaper import latex_tools
import all_paths as ap

INITIALS = "###INITIALS###"

latex_ffp = "%s%s-paper.tex" % (ap.PUBLICATION_PATH, INITIALS)
full_bibtex_ffp = "<path-to-full-bibtex-file>"
local_bibtex_ffp = ap.PUBLICATION_PATH + "%s-references.bib" % INITIALS

clean_first = 1
if clean_first:  # remove month attribute
    a = open(full_bibtex_ffp)
    lines = a.readlines()
    a.close()
    output = []
    for line in lines:
        if "month = " not in line:
            output.append(line)
    ofile = open(full_bibtex_ffp, "w")
    ofile.write("\n".join(output))
    ofile.close()

# citations = latex_tools.extract_citation_keys_from_latex(latex_ffp=latex_ffp)
# citations.sort()
# for citation in citations:
#     print(citation)
p_str = latex_tools.small_bibtex_str(latex_ffp, full_bibtex_ffp, sort=True, verbose=True)
ofile = open(local_bibtex_ffp, "w")
ofile.write(p_str)
ofile.close()




import os

initials = "###INITIALS###"

ROOT_DIR = os.path.dirname(os.path.abspath(__file__)) + "/"
PUB_PATH = ROOT_DIR + "%s_paper/" % initials
FIG_FOLDER = "%s-figures" % initials
PUB_FIG_PATH = PUB_PATH + FIG_FOLDER + "/"  # TODO: change to PUB_FIGURE_PATH
TEMP_FIG_PATH = ROOT_DIR + "temp/"
MODULE_DATA_PATH = ROOT_DIR + "data/"  # no initials, consistent with python science environment from PyCharm

# add project specific paths here

# import user_paths as up

# import user specific paths here


from pypaper import general_tools as gt


def add_init_file(folder_path):
    new_file = open(folder_path + "__init__.py", "w")
    new_file.close()


def build_base_folders(root_folder_full_path, initials):
    # Create folder for latex work
    py_path = "%s%s_scripts/" % (root_folder_full_path, initials)
    gt.build_folder(py_path, "Python")

    # Create images folder and figures folder
    outputs_path = "%soutputs/" % py_path
    gt.build_folder(outputs_path, "Python outputs")
    add_init_file(outputs_path)

    # Create project tools folder
    tools_path = "%stools/" % py_path
    gt.build_folder(tools_path, "Python tools")
    add_init_file(tools_path)
    fname = 'fig_functions.py'
    gt.template_to_folder(fname, tools_path)

    # Create project research folder
    research_path = "%sresearch/" % py_path
    gt.build_folder(research_path, "Python research")
    add_init_file(research_path)

    # Copy figure template file into location
    fname = 'figure_template.py'
    ft_replacements = {"###INITIALS###": initials}
    gt.template_to_folder(fname, outputs_path, ft_replacements)

    # Copy generate_bibtex_file file into location
    fname = 'generate_bibtex_file.py'
    ft_replacements = {"###INITIALS###": initials}
    gt.template_to_folder(fname, outputs_path, ft_replacements)

    # Copy pre-commit hook file into location
    fname = 'pre-commit-hook.sh'
    gt.template_to_folder(fname, root_folder_full_path)
    print("Completed adding base folders")


def add_project_python_files(project_root_folder):
    fname = 'all_paths.py'
    gt.template_to_folder(fname, project_root_folder)
    pass


import re
import bibtexparser
from pypaper import general_tools as gt


def build_base_folders(base_path, root_folder_name, initials, author=""):
    # Create folder for latex work
    latex_path = "%s%s_paper/" % (base_path, initials)
    gt.build_folder(latex_path, "Latex")
    # Create images folder and figures folder
    images_path = "%s%s-images/" % (latex_path, initials)
    gt.build_folder(images_path, "Images")
    figures_path = "%s%s-figures/" % (latex_path, initials)
    gt.build_folder(figures_path, "Figures")
    # Copy base .svg file into image folder

    # Copy latex file into location

    template_ffp = gt.get_template_ffp('base-paper.tex')
    a = open(template_ffp)
    f_str = a.read()
    a.close()
    f_str = f_str.replace("###TITLE###", root_folder_name)
    f_str = f_str.replace("###AUTHOR###", author)
    # Add example .svg file into latex
    latex_file_name = "%s-paper.tex" % initials
    out_ffp = latex_path + latex_file_name
    ofile = open(out_ffp, "w")
    ofile.write(f_str)
    ofile.close()


def find_references(fn):
    """
    Creates a cite key based on a function, or object or method.
    :param fn: function, object or method
    :return:
    """
    full_docstring = fn.__doc__
    ds_lines = full_docstring.split("\n")
    for line in ds_lines:
        if ".. [1]" in line:
            print(line)


def find_equations(fn):
    """
    Creates a latex equation based on a function, or object or method.
    :param fn: function, object or method
    :return:
    """
    full_docstring = fn.__doc__
    ds_lines = full_docstring.split("\n")
    for line in ds_lines:
        if ".. math" in line:
            print(line)


def combine_bibtex(bibtex_ffp1, bibtex_ffp2):
    """
    Merges to bibtex files

    :param ffp:
    :return:
    """

    with open(bibtex_ffp1) as bibtex_file:
        bibtex_database = bibtexparser.load(bibtex_file)

    with open(bibtex_ffp2) as org_bibtex_file:
        org_bibtex_database = bibtexparser.load(org_bibtex_file)

    for entry in bibtex_database.entries_dict:
        if entry not in org_bibtex_database.entries_dict:
            print("Not found: ", entry)
        # else:
        #     print('found')
        # print(entry, bibtex_database.entries_dict[entry])


def compile_bibtex(citations, big_bibtex_ffp):
    """
     finds the bibtex entries that correspond to a list of cite keys,
     from a large bibtex file and writes a new bibtex file
     with just the required references.
     :param citations: a list of citation keys.
    :param big_bibtex_ffp: full file path to bibtex file
    :return:
    """

    remove_keys = ["annote", "date-added", "date-modified", "local-url", "file", "rating", "month", "uri", "read"
                   "abstract", "read"]

    with open(big_bibtex_ffp) as org_bibtex_file:
        org_bibtex_database = bibtexparser.load(org_bibtex_file)

    new_bibtex_db = bibtexparser.loads(" ")  # create a new bibtex DB obj
    for entry in citations:
        if entry not in org_bibtex_database.entries_dict:
            print("Not found: ", entry)
        else:
            new_bibtex_db.entries.append(org_bibtex_database.entries_dict[entry])
            # new_bibtex_db.entries_dict[entry['ID']] = org_bibtex_database.entries_dict[entry]

    # new_bibtex_db = copy.deepcopy(org_bibtex_database)
    # for entry in org_bibtex_database.entries_dict:
    #     if entry not in citations:
    #         print("Not found: ", entry)
    #         del new_bibtex_db.entries_dict[entry]
    #     else:
    #         print("adding: ", entry)
    #         new_bibtex_db.entries_dict[entry] = org_bibtex_database.entries_dict[entry]

    # print(new_bibtex_db.entries)

    for entry in new_bibtex_db.entries_dict:
        for r_key in remove_keys:
            if r_key in new_bibtex_db.entries_dict[entry]:
                del new_bibtex_db.entries_dict[entry][r_key]
    bibtex_str = bibtexparser.dumps(new_bibtex_db)
    return bibtex_str


def extract_citation_keys_from_latex(latex_ffp, chicago=True):
    """
    Reads a latex file and returns a list of cite keys used.

    :param latex_ffp: full file path to latex file
    :return:
    """
    # \\cite\{([^\},]+)(?:,\s*([^\},]+))*\}
    # \\cite\{ = match anything that starts with \cite{
    # (  +) = creates a capture group
    # [^\}] = do not match anything with "}"
    #

    citations = []

    a = open(latex_ffp)
    lines = a.readlines()
    matches = []
    for line in lines:

        matches += re.findall(r'\\cite\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
        matches += re.findall(r'\\citep\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
        matches += re.findall(r'\\citet\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
        matches += re.findall(r'\\citep\[e.g.\]\[\]\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
        if chicago:
            matches += re.findall(r'\\citeN\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
            matches += re.findall(r'\\citeNP\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
            matches += re.findall(r'\\shortcite\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
            matches += re.findall(r'\\shortciteN\{([^\}]+)(?:,\s*([^\},]+))*\}', line)
            matches += re.findall(r'\\shortciteNP\{([^\}]+)(?:,\s*([^\},]+))*\}', line)

    for m in matches:
        for new_cite in m:
            new_cite = new_cite.replace(" ", "")
            if new_cite == "":
                continue
            cites = new_cite.split(",")
            for single_cite in cites:
                if single_cite not in citations:
                    citations.append(single_cite)

    return citations


def small_bibtex_str(latex_ffp, full_bibtex_ffp, sort=True, verbose=False):
    """
    Reads a latex file and extracts the cite keys then finds
     the references in a large bibtex file and writes a new bibtex file
     with just the required references.

    :return:
    """
    citations = extract_citation_keys_from_latex(latex_ffp=latex_ffp)
    if sort:
        citations.sort()
    if verbose:
        for cite in citations:
            print(cite)
        print("Total number of citations: ", len(citations))
    b_str = compile_bibtex(citations, full_bibtex_ffp)
    return b_str


# if __name__ == '__main__':
#     ffp = "../tests/test_data_files/sample_latex.tex"
#     full_bibtex_ffp = "../tests/test_data_files/sample.bib"
#     citations = extract_citation_keys_from_latex(latex_ffp=ffp)
#     print(citations)
#
#     bstr = compile_bibtex(citations, full_bibtex_ffp)
#     print(bstr)



import os
import pypaper


def build_folder(ffp, name):
    """
    Makes a new directory

    :param ffp:
    :param name:
    :return:
    """
    try:
        os.makedirs(ffp, exist_ok=False)
    except OSError:
        print("%s folder already exists" % name)


def build_base_folders(root_folder_full_path, initials):
    """
    Creates the project folders

    :param root_folder_full_path:
    :param initials:
    :return:
    """
    # Create the project temporary output folder
    build_folder(root_folder_full_path + "temp/", "Temporary")
    # Create the project module data output folder
    data_path = "%sdata/" % root_folder_full_path
    build_folder(data_path, "Project data")
    # Copy template files into location
    fname = 'settings.py'
    template_to_folder(fname, root_folder_full_path)
    fname = 'all_paths.py'
    ap_replacements = {"###INITIALS###": initials}
    template_to_folder(fname, root_folder_full_path, ap_replacements)
    fname = 'user_paths.py'
    template_to_folder(fname, root_folder_full_path)
    return root_folder_full_path


def get_template_ffp(template_name):
    """
    Gets the full path to the template

    :param template_name:
    :return:
    """
    templates_dir = os.path.join(os.path.dirname(pypaper.__file__), 'templates')
    template_file = os.path.join(templates_dir, template_name)
    return template_file


def template_to_folder(template_name, folder_path, replacements={}):
    """
    Copies a template into a folder

    :param template_name:
    :param folder_path:
    :return:
    """
    template_ffp = get_template_ffp(template_name)
    a = open(template_ffp)
    f_str = a.read()
    a.close()
    for key in replacements:
        f_str = f_str.replace(key, replacements[key])
    out_ffp = folder_path + template_name
    ofile = open(out_ffp, "w")
    ofile.write(f_str)
    ofile.close()


def add_to_gitignore(root_folder_full_path, initials):
    """
    Add a list of files that should not be tracked by git

    :param base_path:
    :param initials:
    :return:
    """
    # Find gitignore file
    to_ignore = ["# Added by pypaper",
                 ".idea/",
                 "user_paths.py",
                 "*.pyc",
                 "temp_output/",
                 "%s_scripts/__pycache__/" % initials,
                 "%s_scripts/matplotlibrc" % initials,
                 "*TSWLatexianTemp_*",
                 "%s-paper/%s-paper.pdf" % (initials, initials),
                 "temp/*",
                 "venv/"
                 ]
    a = open(root_folder_full_path + ".gitignore", "a")
    a.write("\n" + "\n".join(to_ignore))
    a.close()


def add_to_requirements_txt(root_folder_full_path):
    """
    Adds python packages used in project

    :param project_root_folder:
    :return:
    """
    to_ignore = ["matplotlib",
                 "bwplot",
                 "engformat>=0.1.3"

                 ]
    a = open(root_folder_full_path + "requirements.txt", "a")
    a.write("\n" + "\n".join(to_ignore))
    a.close()



import sys
import os

# from pypaper import MIMUMUM_VERSION
import pypaper
from pypaper import general_tools as gt
from pypaper import latex_tools as lt
from pypaper import py_tools as pyt

__project_pypaper_version__ = "0.2.6"  # version that project was built with
PROJECT_ROOT_FULL_PATH = os.path.dirname(os.path.abspath(__file__)) + "/"
PROJECT_NAME = os.path.basename(os.path.dirname(os.path.abspath(__file__)))

import sys
from distutils.version import StrictVersion

if StrictVersion(__project_pypaper_version__) < StrictVersion(pypaper.MINIMUM_VERSION):
    raise ValueError("project built with incompatible pypaper version: %s; "
                     "required version: %s; "
                     "Or install an older version of pypaper." % (__project_pypaper_version__,
                                                                  pypaper.MINIMUM_VERSION))


def new_research_project(initials):
    print("building research project")
    root_folder_full_path = PROJECT_ROOT_FULL_PATH
    project_name = PROJECT_NAME

    print(folder_path, initials)
    gt.build_base_folders(root_folder_full_path, initials)
    lt.build_base_folders(root_folder_full_path, project_name, initials, author="")
    pyt.build_base_folders(root_folder_full_path, initials)
    gt.add_to_gitignore(root_folder_full_path, initials)
    gt.add_to_requirements_txt(root_folder_full_path)
    pass


def get_script_path():
    ffp = os.path.dirname(os.path.realpath(sys.argv[0])) + "/"
    return ffp


def build_new_bibtex(latex_ffp, big_bibtex_ffp, new_bibtex_ffp='new_bibtex.bib'):
    citations = lt.extract_citation_keys_from_latex(latex_ffp=latex_ffp)
    print(citations)
    bstr = lt.compile_bibtex(citations, big_bibtex_ffp)
    with open(new_bibtex_ffp, 'w') as bibfile:
        bibfile.write(bstr)


def console_help():
    print("Usage: python manage.py <option> <args>")
    print("Options: init <optional: initials>")
    print("         init <optional: initials>")
    print("         short-bib <latex file path> <large bibtex path> <optional: new bibtex path>")


if __name__ == "__main__":
    try:
        arg1 = sys.argv[1]
    except IndexError:
        console_help()
        sys.exit(1)
    folder_path = get_script_path()

    # start the program
    if arg1 in ["init"]:
        if len(sys.argv) < 3:
            # prompt user
            initials = "ni"
            pass
        else:
            initials = sys.argv[2]
        new_research_project(initials)
    elif arg1 == "short-bib":
        try:
            latex_ffp = sys.argv[2]
            big_bibtex_ffp = sys.argv[3]
        except IndexError:
            console_help()
            sys.exit(1)
        if len(sys.argv) < 5:
            new_bibtex_ffp = 'new_bibtex.bib'
        else:
            new_bibtex_ffp = sys.argv[4]
        build_new_bibtex(latex_ffp, big_bibtex_ffp, new_bibtex_ffp)

    else:
        console_help()


import matplotlib.pyplot as plt
from bwplot import cbox
import engformat as ef

from ###INITIALS###_scripts.tools import fig_functions as ff
import all_paths as ap

import settings as ops


def create(save=0, show=0):

    bf, subplot = plt.subplots(figsize=(ops.TWO_COL, 4))
    subplot.plot([0], [0], label="", c=cbox(0))

    subplot.set_xlabel('Period [s]')
    # subplot.set_xscale('log')
    subplot.set_ylabel('Disp. Response [m]')
    ef.xy(subplot, x_origin=True, y_origin=True)
    ef.revamp_legend(subplot, loc="upper left", prop={'size': 9})

    bf.tight_layout()
    name = __file__.replace('.py', '')
    name = name.split("figure_")[-1]
    extension = ""
    if save:
        bf.savefig(ap.PUB_FIG_PATH + name + extension + ops.PUB_FIG_FILE_TYPE, dpi=ops.PUB_FIG_DPI)
        if ops.PUB_DOCUMENT_TYPE == "latex":
            para = ef.latex_for_figure(ap.FIG_FOLDER, name=ap.PUB_FIG_PATH + name + extension, ftype=ops.PUB_FIG_FILE_TYPE)
            print(para)
    if show:
        plt.show()


if __name__ == '__main__':
    create(save=0, show=1)
